
#####################################################################################################
##
##      NANP Modeling Workshop 2021 - Compartmental Model Design, fitting, and parameter estimation
##      Kristan Reed, Cornell University
##
#####################################################################################################


### An example model of rumen protein N digestion, microbial growth and flow to the duodenum that will
### be solved with the ode() function in the deSolve package followed by parameter optimization with the 
### modFit() function of the FME package

### Load required packages
library(deSolve)
library(FME)

#####################################################################################################
##
##    Step 1: Specify Initial conditions
##
#####################################################################################################

## Defining a reasonable number for the value of each state variable at time t=0 is an important first
## step in solving initial value problems. Setting real numbers for the initial values allows the ODE
## system to be solved explicitly. 
## Here I use a typical naming convention where all state variables are given a prefix of the capital
## letter 'Q', followed by an abbreviation for each state pool

Initials = c( QDegN = 120, ## The QDegN pool (g N) represents the pool of potentially degradable N in the rumen
                            ##  This is intended to be analogous to the Protein B fraction in NRC (2001)
              QSolN = 60, ## The QDegN pool (g N) represents the pool of soluble N in the rumen
                            ##  This is intended to be analogous to the Protein A fraction in NRC (2001)
              QMicN = 10)  ## The QMicN (g N) pool is the pool of microbial N

#####################################################################################################
##
##    Step 2: Specify the Parameters
##
#####################################################################################################

## The parameter vector can include any parameters that are in the model however, if the intent is to 
## optimize the model for a specific set of parameters, only those parameters that you intend to fit 
## to the data should be included in the vector that is passed to the ODE function. 
## Other parameters needed by the model can be fixed within the ODE function

params = c(kd = 0.13, #Rate of degradation of degradable N into soluble N (units = (1/hr))
  km = 3, #Michaelis constant which is equal to the substrate concentration at which 
          # microbial protein production is 1/2 of Vmax (units = g SolN/l)
  Vmax = 9.5, #the maximum rate of microbial protein production (units = g MicN / hr)
  kp = 0.09) #the rate of passage of digesta out of the rumen (units = (1/hr))



#####################################################################################################
##
##    Step 3: Specify the the time sequence for which you want to solve the system of equations
##
#####################################################################################################

## Here we define a vector for 1 day and 1 hour at a time step of 1 hour to reflect our data

Times = seq(0, 25, by = 1)


#####################################################################################################
##
##    Step 4: Define the ODE system as a new function
##
#####################################################################################################

## If you haven't created your own R function before, please take a moment to look at the R_Help.R file

## The function that describes the ODE system needs to include a specific set of inputs in order for it to 
## be solved by the ode() function. The inputs include: 
    ##  a vector of time points at which the solver will output the solutions of the system
    ##  a vector of the state variables and their initial values at time t=0
    ##  a vector of parameters with estimates the modeler believes to be close to their real value

Rumen_N <- function(t, state, parm){ ## note that the variable names used by the function are not exactly
                                    ## the same as the variable names used to describe the objects that 
                                    ## will be used as inputs
  
  with(as.list(c(state,parm)),   ## this statement is not strictly necessary but enables the function 
                                 ## statements below to reference the names of the variables within 
                                 ## the state and parm inputs without indexing the state and parm objects
                                 ## For example, without the with statement, FDegSol would need to be 
                                 ## defined as: parm['kd]*state['QDegN] instead of kd*QDegN as below
                                
       {
         ## As with most R code, all parameters definitions must be sequentially ordered since it is not
         ## compiled. Before a parameter is used, it must be defined
         
         ## Define any additional parameters
         VolR = 50 # This is a parameter for rumen volume (liters) that is not included in the parm vector
                   # because we assume it was measured and thus will not be fit to the data
         
         
         ## Build equations for flows in and out of the state pools
         ## The convention that I use for naming flows is:
            ## Start with prefix 'F'
            ## Follow with abbreviation for the pool from which the flow originates
            ## End with the abbreviation for the destination pool or outflow
         
         FDegSol = kd*QDegN #flow of degradable N to Soluble N (i.e. protein degradation)
         FSolMic = Vmax/(1 + km/(QSolN/VolR)) # flow of soluble N to microbial N (i.e. MicN synthesis)
         FSolDuod = kp*QSolN # flow of soluble N to the duodenum
         FDegDuod = kp*QDegN # flow of degradable N to the duodenum
         FMicDuod = kp*QMicN # flow of microbial N to the duodenum
         
         
         #Define the ODE system with one equation for each state pool
         #Note that the name for the derivative of the state must follow the
         #specific annotation specified by the ode system which is: 
         # a small 'd' followed by the variable name for the state variable
         dQDegN =  - FDegSol - FDegDuod
         dQSolN =  FDegSol - FSolMic - FSolDuod
         dQMicN = FSolMic - FMicDuod
         
         
         # define a list of outputs that you wish to see the values for at each time point
         # Note that the first output vector must be the list of the state variables,
         # after that you can return whatever variable, flux, etc you like
         list(c(dQDegN, dQSolN, dQMicN),c(dQSolN = dQSolN, dQDegN = dQDegN, dQMicN= dQMicN))
       })
  
}



#####################################################################################################
##
##    Find the solutions to the system with the ode() function
##
#####################################################################################################

    #By default, ode() returns a matrix. I find it easier to work with dataframes for
    #plotting but it is not necessary to transform it to a data frame

output = data.frame(ode(y = Initials, times = Times, func = Rumen_N, parms = params ))

#Look at the results - what do each of the columns represent?
output


## you can use any plotting function you prefer to plot the outcomes, I like to use matplot as a 
## quick way to see multiple outcomes over time

matplot(x = output$time, y = output[ , 2:7], type="l", col = 1:3, lty = c(rep(1, 3), rep(2, 3)), 
        xlab = "Time (h)", ylab = "Solutions (g N)")
legend(x = 15, y = 125, legend = c("DegN", "SolN", "MicN", "Q", "dQ"), fill= c(1, 2, 3, NA, NA), 
       bty = "n", lty = c(NA, NA, NA, 1, 2), border = NA, cex = 0.6)



#All of the pools and flows are approaching 0 - why?

#####################################################################################################
##
##    Step 6: Build a function to determine the criteria for parameter optimization
##
#####################################################################################################

##This step is to determine how the optimization function will determine which parameter estimates are
## better than others. The modCost() function estimates the squared residuals which is a common
## choice for optimization to minimize the 'sum of squares' or the sum of the squared residuals

Cost<- function(P, Inits, Func, Obs, obstime, Costs = NULL)
{
  #This function requires similar inputs to ode() becuase it will be running the ode function
  #In addition to the initial states, the ODE system function, and the parameters to optimize, 
  #it also requires the observed data that the parameters are to be optimized for, and the time
  #points at which the data was observed
  #P - a vector of the parameters and their values - *Note that this must be first
  #Inits - list of initial values for the states
  #Func - the function describing the ode system
  #Obs - a vector or list of the observed data
  #obstime - a vector of the timepoints for the observed data 
  #Costs - the modCost output from another observed variable to be combined with the current obs
  
  #get the solutions to compare to the observed data
  out=ode(y = Inits, times=obstime, func = Func, parms=P)
  
  #get the model sums of squared residuals
  Cost=modCost(model=out, obs=Obs, cost = Costs)  
  
  return(Cost)
}

#####################################################################################################
##
##    Step 7: Load and format data
##
#####################################################################################################


Observed = read.csv('Example_Data_Set_share.csv')
  
head(Observed)

#Rename column names - can be done in excel...user preference
names = c('Time', 'Animal.ID', 'NI', 'NH3Conc', 'NH3', 'BacMassConc', 'BacMass', 'BacN', 'DuodDigestFlow', 'DuodNComp', 'DuodNFlow', 'DuodNFlow_hr', 
         'DuodNH3Comp', 'DuodNH3-Ncomp', 'DuodBacN-Ncomp' )

colnames(Observed) = names
head(Observed)

### Which values in this data set can be used to fit our model? Which model output correlates to the available data?



####### Step 7.1 Create a vector of observed data time points
        ##  Notice that the measured values of interest are not available at every time point in the dataset and 
        ## that the time points are repeated for each animal and that time t=0 is when the first rumen sample was taken
  Observed = Observed[!is.na(Observed$BacN), ]
  Time.obs = Observed$Time[Observed$Animal.ID == 1]

###### Step 7.2 Create a dataframe or matrix of the observed values in the correct format for the modCost function
        ## modCost can take two different forms - long or wide. We are starting with observed data for one model output 
        ## so it doesn't matter much. 
        ## It is important to match the names of the observed data to the model outputs that will be compared to the obs data.
        ## in this dataset, BacN in the observed data correlates to the model MicN


  
    ##  Wide Format
  Obs.dat.wide =Observed[ , c('Time', 'BacN')]
  colnames(Obs.dat.wide) = c('time', 'QMicN')
  
  head(Obs.dat.wide)
  
  ## Step 7.3 Test time vector and fitting data objects in the cost function
  
  # Find model predicted solutions 
  out = ode(y = Initials, func = Rumen_N, parms = params, times = Time.obs)
  # pass the system solutions and observed data to modCost
  C = modCost(model = out, obs= Obs.dat.wide)
  C
  
  ## What are the outputs of the modCost funtion?
  

  ## Do you get the same output if you use the function that we built earlier?
  
  Cost(P =  ,Inits =  , Func =  ,  Obs =  , obstime = )
  


#####################################################################################################
##
##    Step 8: Optimize parameters to minimize model costs
##
#####################################################################################################

## The modFit() function searches for the parameters that minimize the model costs.
## The inputs to modFit() include: 
  ## f - the function to minimize, in our case, Cost()
  ## p - the vector of parameters and their initial search values
  ## lower, upper - lower and upper bounds for the parameters; cab be set individually or all at once
  ## the optimization method
  ## all of the inputs needed for the function input to f
  


fit<- modFit( f = Cost, p=params, lower=0, upper = 10,  method="Nelder-Mead", 
             Inits = Initials, Func = Rumen_N,  Obs = Obs.dat.wide, obstime = Time.obs)

summary(fit)
#Do these parameters make sense?

#What does it mean that the parameters are highly correlated to eachother?



### Get the model solutions with the new fitted parameter values
output_fit = data.frame(ode(y = Initials, parms = fit$par, times = Times, func = Rumen_N))
head(output_fit)


matplot(x = output_fit$time, y = output_fit[ , 2:7], type="l", col = 4:6, lty = c(rep(1, 3), rep(2, 3)), 
        xlab = "Time (h)", ylab = "Solutions (g N)")
legend(x = 15, y = 125, legend = c("DegN", "SolN", "MicN", "Q", "dQ"), fill= c(4, 5, 6, NA, NA), 
       bty = "n", lty = c(NA, NA, NA, 1, 2), border = NA, cex = 0.6)

matplot(x = Times, y = output_fit[, 4], type = "l",ylim = c(0, 50) , 
        ylab = "MBN (g N)", xlab = "Time, hr")
points(QMicN~ time, dat = Obs.dat.wide, col = 2, pch = 19)

fit$par



### What happens if you change the parameter starting values in params?

params = c(kd = , km = , Vmax = , kp = )




### What happens if you change the boundaries?



### What happens if you change the Initial values?






