setwd('C:/users/kfr3/documents/conferences/ADSA 2021/NANP 2021/')
#####################################################################################################
##
##      NANP Modeling Workshop 2021 - Compartmental Model Design, fitting, and parameter estimation
##      Part 2: Fitting to additional data and data types
##      Kristan Reed, Cornell University
##
#####################################################################################################


### We will continue with the rumen N model example and our dataset to see if we can improve parameter estimates
### by fitting to more data

### Load required packages
library(deSolve)
library(FME)


### start with the same steps... 

#####################################################################################################
##
##    Step 1: Specify Initial conditions
##
#####################################################################################################
Initials = c( QDegN = 120, ## The QDegN pool (g N) represents the pool of potentially degradable N in the rumen
              ##  This is intended to be analogous to the Protein B fraction in NRC (2001)
              QSolN = 60, ## The QDegN pool (g N) represents the pool of soluble N in the rumen
              ##  This is intended to be analogous to the Protein A fraction in NRC (2001)
              QMicN = 20)  ## The QMicN (g N) pool is the pool of microbial 
#####################################################################################################
##
##    Step 2: Specify the Parameters
##
#####################################################################################################


params = c(kd = 0.13, #Rate of degradation of degradable N into soluble N (units = (1/hr))
           km = 3, #Michaelis constant which is equal to the substrate concentration at which 
           # microbial protein production is 1/2 of Vmax (units = g SolN/l)
           Vmax = 9.5, #the maximum rate of microbial protein production (units = g MicN / hr)
           kp = 0.09) #the rate of passage of digesta out of the rumen (units = (1/hr))



#####################################################################################################
##
##    Step 3: Specify the the time sequence for which you want to solve the system of equations
##
#####################################################################################################

Times = seq(0, 25, by = 1)


#####################################################################################################
##
##    Step 4: Build ODE model - this time add additional outputs to the ODEsystem function
##
#####################################################################################################

Rumen_N <- function(t, state, parm){ ## note that the variable names used by the function are not exactly
  ## the same as the variable names used to describe the objects that 
  ## will be used as inputs
  
  with(as.list(c(state,parm)),   ## this statement is not strictly necessary but enables the function 
       ## statements below to reference the names of the variables within 
       ## the state and parm inputs without indexing the state and parm objects
       ## For example, without the with statement, FDegSol would need to be 
       ## defined as: parm['kd]*state['QDegN] instead of kd*QDegN as below
       
       {
         ## As with most R code, all parameters definitions must be sequentially ordered since it is not
         ## compiled. Before a parameter is used, it must be defined
         
         ## Define any additional parameters
         VolR = 50 # This is a parameter for rumen volume (liters) that is not included in the parm vector
         # because we assume it was measured and thus will not be fit to the data
         
         
         ## Build equations for flows in and out of the state pools
         ## The convention that I use for naming flows is:
         ## Start with prefix 'F'
         ## Follow with abbreviation for the pool from which the flow originates
         ## End with the abbreviation for the destination pool or outflow
         
         FDegSol = kd*QDegN #flow of degradable N to Soluble N (i.e. protein degradation)
         FSolMic = Vmax/(1 + km/(QSolN/VolR)) # flow of soluble N to microbial N (i.e. MicN synthesis)
         FSolDuod = kp*QSolN # flow of soluble N to the duodenum
         FDegDuod = kp*QDegN # flow of degradable N to the duodenum
         FMicDuod = kp*QMicN # flow of microbial N to the duodenum
         
         
         
         #Define the ODE system with one equation for each state pool
         #Note that the name for the derivative of the state must follow the
         #specific annotation specified by the ode system which is: 
         # a small 'd' followed by the variable name for the state variable
         dQDegN =  - FDegSol - FDegDuod
         dQSolN =  FDegSol - FSolMic - FSolDuod
         dQMicN = FSolMic - FMicDuod
         
         
         ## Create a flux that is the sum of the flows out of the duodenum
         FRumenDuod = FSolDuod + FDegDuod + FMicDuod
         
         
         ## Create a concentration variable
         SolNConc = QSolN/VolR
         
         # define a list of outputs that you wish to see the values for at each time point
         # Note that the first output vector must be the list of the state variables,
         # after that you can return whatever variable, flux, etc you like
         list(c(dQDegN, dQSolN, dQMicN),c(dQSolN = dQSolN, dQDegN = dQDegN, dQMicN= dQMicN, 
                                          DuodNFlow= FRumenDuod, NH3Conc = SolNConc, 
                                          FMicDuod = FMicDuod))
       })
  
}


#####################################################################################################
##
##    Step 6: Build a function to determine the criteria for parameter optimization
##
#####################################################################################################
Cost<- function(P, Inits, Func, Obs, obstime, Costs = NULL)
{
  #This function requires similar inputs to ode() becuase it will be running the ode function
  #In addition to the initial states, the ODE system function, and the parameters to optimize, 
  #it also requires the observed data that the parameters are to be optimized for, and the time
  #points at which the data was observed
  #P - a vector of the parameters and their values - *Note that this must be first
  #Inits - list of initial values for the states
  #Func - the function describing the ode system
  #Obs - a vector or list of the observed data
  #obstime - a vector of the timepoints for the observed data 
  #Costs - the modCost output from another observed variable to be combined with the current obs
  
  #get the solutions to compare to the observed data
  out=ode(y = Inits, times=obstime, func = Func, parms=P)
  
  #get the model sums of squared residuals
  Cost=modCost(model=out, obs=Obs, cost = Costs)  
  
  return(Cost)
}

#####################################################################################################
##
##    Step 7: Load and format data
##
#####################################################################################################


    Observed = read.csv('Example_Data_Set_share.csv')
    
    head(Observed)
    
    #Rename column names - can be done in excel...user preference
    names = c('Time', 'Animal.ID', 'NI', 'NH3Conc', 'NH3', 'BacMassConc', 'BacMass', 'BacN', 'DuodDigestFlow', 'DuodNComp', 'DuodNFlow', 'DuodNFlow_hr', 
              'DuodNH3Comp', 'DuodNH3-Ncomp', 'DuodBacN-Ncomp' )
    
    colnames(Observed) = names
    head(Observed)
    

    
####### Step 7.1 Create a vector of observed data time points
  
  Observed = Observed[!is.na(Observed$BacN), ]
  Time.obs = Observed$Time[Observed$Animal.ID == 1]

###### Step 7.2 Create a dataframe or matrix of the observed values in the correct format for the modCost function


  ##  Wide Format
  Obs.dat.wide2 =Observed[ , c('Time', 'BacN', 'DuodNFlow_hr')]
  colnames(Obs.dat.wide2) = c('time', 'QMicN', 'DuodNFlow') #note to change the name to match the variable name in Rumen_N
  
  head(Obs.dat.wide2)

## Step 7.3 Test time vector and fitting data objects in the cost function

  # Find model predicted solutions 
  out = ode(y = Initials, func = Rumen_N, parms = params, times = Time.obs)
  # pass the system solutions and observed data to modCost
  C = modCost(model = out, obs= Obs.dat.wide2)
  C


#####################################################################################################
##
##    Step 8: Optimize parameters to minimize model costs
##
#####################################################################################################



  fit2<- modFit( f = Cost, p=params, lower=0, upper = 20,  method="Nelder-Mead", 
                Inits = Initials, Func = Rumen_N,  Obs = Obs.dat.wide2, obstime = Time.obs)
  
  summary(fit2)
#Do these parameters make sense?

#How does this model cost compare to just fitting to one set of data?



### Get the model solutions with the new fitted parameter values
output_fit2 = data.frame(ode(y = Initials, parms = fit2$par, times = Times, func = Rumen_N))
head(output_fit2)

### Plot model solutions and observed data
matplot(x = output_fit2$time, y = output_fit2[ , 2:8], type="l", col = c(4:6,4:6,7),
        lty = c(rep(1, 3), rep(2, 3), 1), 
        xlab = "Time (h)", ylab = "Solutions (g N)")
legend(x = 15, y = 125, legend = c("DegN", "SolN", "MicN","DuodNFlow", "Q", "dQ"), fill= c(4, 5, 6, 7, NA, NA), 
       bty = "n", lty = c(NA, NA, NA,NA, 1, 2), border = NA, cex = 0.6)
points(QMicN~ time, dat = Obs.dat.wide2, col = 6, pch = 19)
points(DuodNFlow~time, dat = Obs.dat.wide2, col = 7, pch = 19)

fit2$par



#####################################################################################################
##
##    Step 8: What happens if use more data?
##
#####################################################################################################



## Create a vector of Bacterial N flow to the duodenum
Observed$BacNDuodFlow = Observed$DuodNFlow_hr

## Create a data in the format for modCost
Obs.dat.wide3 = Obs.dat.wide2
Obs.dat.wide3$FMicDuod =Observed$BacNDuodFlow


Cost1 = modCost(model = out, obs= Obs.dat.wide2)
Cost1
modCost(model = out, obs= Obs.dat.wide3, Cost = Cost1)



fit3<- modFit( f = Cost, p=params, lower=0, upper = 20,  method="Nelder-Mead", 
               Inits = Initials, Func = Rumen_N,  Obs = Obs.dat.wide3, obstime = Time.obs)

summary(fit3)



### Get the model solutions with the new fitted parameter values
output_fit3 = data.frame(ode(y = Initials, parms = fit3$par, times = Times, func = Rumen_N))
head(output_fit3)

### Plot model solutions and observed data



### How do the parameter estimates differ when fit to different data? How would you handle this if 
### it happened in your own research?
