################################# NANP Nutrition Models Workshop #################################

# ADSA 2021 Annual Meeting
# Automated model selection (AMS): Part II (exercises) 
# Dr. Veridiana L. Daley Email: veridianalsouza@gmail.com

# Objective: Apply the AMS approach as a tool for the selection of prediction models.
# The dataset used in this lesson can be downloaded from the NANP website (Dairy digestion, https://animalnutrition.org/modeling-database)

# Tips:
      # Use "#" to make comments (green letters).
      # In the "script" the codes are in black and blue.
      # In the "console" the codes are in blue and results in black.
      # Executing code (two options): 
        #1. Select the code and press the Ctrl+Enter.
        #2. Select the code and press the Run toolbar button (on the top right side of this script).
      # Clear the Console press Ctrl+L
      # Restart the RStudio session  Ctrl+Shift+F10


# If you are a new RStudio user, please read the R manual on the NANP website
# Link: https://animalnutrition.org/sites/default/files/2018-06/Installing%20and%20using%20R%20updated%207.26.2017.pdf


################################  Lesson 3 #########################################################


################ Part 1: Import Dataset ################
options(scipen = 999)

# Clean up your environment on RStudio.
    rm(list = ls(all = TRUE)) # Delete all R object
    graphics.off() # Delete all open graphics 

# Verify your working directory
    getwd()
 
# If necessary, set your working directory
    #setwd("C:/Users/verid/Desktop/2021 ADSA/Codes")
    
# Read the data (CSV file) 
    library(readr)   # Load the library necessary to open the data.CSV file (make sure that this library was installed previously, and run the code twice to make sure that it is in blue color on the Console)
    
    # Option A: Load the dataset named "data" (from your .Rproj)
    data <- read_csv("Exercise #3 data file.csv") # 658 observations and 26 variables (see it at the "Global Environment" on the top right of the your screen))
    
    # Option B: Go to "Environment" (on right) > Import Dataset > From Text (readr) > File > data.CSV (USB drive) > Open > Import

################ Part 2: Data Cleaning ###################

# Show the names of all variables 
    ls()   # Fill out here

# Summary of the dataset
    summary() # Fill out here
   
# Summary table 
library(summarytools)
    view(dfSummary(data)) # read the results on Viewer

# Rename an observation with misspelling case
    data$An_BW <- ifelse(data$An_BW == , , data$An_BW)  # Fill out here

# See studies with missing BW
    missing_BW <- data[is.na(data$An_BW),]
    missing_BW
    remove(missing_BW)
  
# Remove studies (PubIDS) with missing BW data
    data <- subset(data, data$PubID !=  & data$PubID != ) # Fill out here
 
# Remove variables  
    ls(data) 
    data$X1 <- NULL 
    data$TrtName <- NULL  
    data$Obs_MilkLacp <- NULL
    
# Remove missing data
    data <- subset(data, data$Obs_MilkPrtp != is.na(data$Obs_MilkPrtp))
   
# Add new variables to a data frame
    data$An_BWm <- data$An_BW ^ # calculate metabolic body weight 

# Create a copy of the database and name it as "d" 
    d <- data # See two data.frames on the Global Environment 
    
# Check column classes and changing it to "numeric" if necessary
    class(d$An_BW) # one variable
    sapply(d, class) # all variables
    
    # If necessary to change the variable class to numeric, use the code example: data$variablename <- as.numeric(data$variablename)
    # If necessary to change the variable class to character, use the code example: data$variablename <- as.character(data$variablename)

################### Part 3: Plots and Remove Outliers ###################
    library(reshape2) # Run the library
    library(ggplot2)  # Run the library
    
    # Plot 1: Plot of data to identify outliers
    mlt <- d[c("PubID", "TID", "An_DMIn", "An_BW", "An_LactDays", "Dt_ADF")] # fill out using variables of the dataset 
    mlt <- melt(mlt, id="TID")
    p1 <- ggplot(mlt, aes(x=value))+geom_density(alpha = 0.2)+facet_wrap(~variable, ncol=6, scales="free") + theme(strip.text = element_text(size = 20, color = "white", hjust = 0.5,vjust = 0.5, face = "bold"),                                                                                                              strip.background = element_rect( fill = "deepskyblue4", color = NA )) + theme(axis.text=element_text(size=15), axis.title=element_text(size=15,face="bold"))
    p1 # See plots on Plots (on right side)
    
    # Plot 2: Histogram DMI in kg/day
    p2 <- ggplot(d, aes( )) + geom_histogram(color="red", fill="red", alpha = .5, position = "identity") + theme(axis.text=element_text(size=18), axis.title=element_text(size=18,face="bold"))
    p2           # Fill out here variable name
    
    
    # Models for cows 100 to 300 DIM
    d <- subset(d, d$An_LactDays >100 & d$An_LactDays <=300) # Fill out here
    summary(d$An_LactDays)
    
    # Plot 3: Plot dry matter intake (An_DMIn, kg/d) and BW
    p3 <- ggplot(d, aes(x = An_BW, y = An_DMIn)) + stat_smooth(method = "lm", size = 1)
    + geom_point()
    p3

    # Save plot
    library(png)
    ggsave(p1, file="Figure 1.jpeg", height= 11, width= 20, dpi= 400)
    ggsave(p2, file="Figure 2.jpeg", height= 11, width= 20, dpi= 400)
    ggsave(p3, file="Figure 3.jpeg", height= 11, width= 20, dpi= 400)
    
    # Remove remove outliers using the standard deviations  (do it to all candidate variables)
    # Variable DMI
    summary(d$An_DMIn)
    
    upper <- mean(d$An_DMIn) + (2.5*sd(d$An_DMIn))  # cutoff 3 standard deviations from the mean (apply the sd function)
    upper
    lower <- mean(d$An_DMIn) - (2.5*sd(d$An_DMIn))  # cutoff 3 standard deviations from the mean (apply the sd function)
    lower
    
    d <- d[which(d$An_DMIn > lower & d$An_DMIn < upper), ]
    summary(d$An_DMIn)

################### Part 4:  Verify the final dataset and Export  ################### 

    ls(d) # 463 observations and 24 variables

    # Check missing data
    sapply(d, function(x) sum(is.na(x)))
    

    # Split Data into Training (0.75) and Testing (0.25) in R 
    library(plyr)
   (count(d$PubID)) # n= 116 studies
    
    # R sample - generate random sample in r for evaluation database
    set.seed(123)    # Set seed for reproducibility
    id = unique(d$PubID)
    sample(id,size=)  # fill out here (25% of 116 studies)
    
    # Database for Evaluation (25%)
    library(plyr)
    test = subset(d, d$PubID==112|d$PubID==315|d$PubID==167|
                   d$PubID==30|d$PubID==205|d$PubID==139|
                   d$PubID==166|d$PubID==141|d$PubID==368|
                   d$PubID==390|d$PubID==103|d$PubID==334|
                   d$PubID==352|d$PubID==208|d$PubID==371|
                   d$PubID==185|d$PubID==353|d$PubID==18|
                   d$PubID==355|d$PubID==305|d$PubID==104|
                   d$PubID==16|d$PubID==388|d$PubID==366|
                   d$PubID==323|d$PubID==120|d$PubID==314|
                   d$PubID==320|d$PubID==386)
    (count(test$PubID)) # n= 29 studies
    
    # Database for Model development (training, 75%)
    
    ### Function to subset using 3 ids
    subset_func<- function(df, x,y,z) {
      df = subset(df, df$PubID !=x)
      df = subset(df, df$PubID !=y)
      df = subset(df, df$PubID !=z)
      (df) }
    
    train = subset_func(d,112,315,167)
    train = subset_func(train,30,205,139)
    train = subset_func(train,166,141,368)
    train = subset_func(train,390,103,334)
    train = subset_func(train,352,208,371)
    train = subset_func(train,185,353,18)
    train = subset_func(train,355,305,104)
    train = subset_func(train,16,388,366)
    train = subset_func(train,323,120,314)
    train = subset_func(train,320,386,386)
    (count(train$PubID)) # 87 studies

    # Save final database
    write.csv(train, "train database  n= 338.csv",na="",row.names=FALSE)
    write.csv(test, "test database  n= 125.csv",na="",row.names=FALSE)
    
  # Make sure that both database (train and test are saved on Files (on the right side))
    
#################################################################################################
#                              Part 5: Automated model selection 
################################################################################################# 
 
    # Clean up your environment on RStudio.
    rm(list = ls(all = TRUE)) # Delete all R object
    graphics.off() # Delete all open graphics 
   
  
    # Read the cleaned data (CSV file) 
    library(readr)   # Load the library necessary to open the data.CSV file (make sure that this library was installed previously, and run the code twice to make sure that it is in blue color on the Console)
    d <- read_csv("train database  n= 338.csv") # 338 observations and 24 variables (see it at the "Global Environment" on the top right of the your screen))
    ls(d)
    
   
################### 1. Parallel Computing in RStudio ###########
library(parallel) # Enable multiple computations to take place at the same time.
                  # 
   # Find out how many cores your processor has
    num_cores <- (detectCores())		# All cores
    num_cores
    
   # Calculate the number of cores to use
    num_cores <- (detectCores() - 1)	
    num_cores
    
    # Initiate cluster
    cl <- makeCluster(4)	
    
    # Load packages on the cluster named cl
    clusterEvalQ(cl, library(lme4)) 
    clusterEvalQ(cl, library(MuMIn))
    
    # Load the dataset named d on the cluster
    clusterExport(cl, "d")
    
###################  2. Describe the global mixed model using all potential predictor variables ###########
    
    library(lme4) # run the library
    library(MuMIn) # run the library
    library(lmerTest)
    # Fit the Global Model (include all independent variables that potentially affect the DMI) 

    m1_global <- lmer(An_DMIn ~ Obs_MilkProd + An_LactDays + An_BW + An_BWm + Dt_Forage + Dt_St + Dt_FA +
                        (1|PubID), data=d, weights = sqrt(N_study), REML=FALSE) # Linear mixed model fit by maximum likelihood
    summary(m1_global)
    #warnings()
   
################### 3. Derive a Set of Candidate Models ###################

    # Use the "dredge" function to generate models using combinations of the terms in the global model. 
    # The function will also calculate AICc values and rank models according to it. 

    
    # Set up "NA action"
    options(na.action = "na.fail") 
    
    # Set up required independent variables
    ReqVars <- c("An_BW") # fixed = ReqVars
    
    # Use the dredge function to fit models
    allmods <- pdredge(m1_global, cluster = cl, rank = "AICc", trace=2, m.lim=c(0, 20))
    #allmods <- pdredge(m1_global, cluster = cl, rank = "AICc", fixed = "An_BW", trace=2, m.lim=c(0, 20))
    warnings()
    
    # Return the "NA action"
    options(na.action = "na.omit") 
    
    # Open the set of candidate models
    View(allmods)

    # Stop cluster after finish
    stopCluster(cl)     
        
    #Question: How many models were fitted? _______________
    
  
################### 4. Collecting Results  ###################    

    # Create two datasets of candidate models       
    allmods <- allmods # all candidate models are collected
    Best20 <- head(allmods,20) # 20 models with the lowest AICc values are collected
    
    # Save the datasets in your working directory (computer)
    save(allmods, file=" ") # fill out here
    save(Best20, file="Best20 df.Rda")

    # Save the dataset as .CSV n your working directory (computer)
    write.csv(allmods, file = "") # fill out here
    write.csv(Best20, file = "Best20.csv")

    # Load the models from rda file
     # load(file = "allmods.rda")
     # head(allmods,20)

################### 5. Comparisons  ###################

    library(lme4) # Run the library
    library(MuMIn) # Run the library

    # Save coefficients in a table
    avgBest20 <- coefTable(Best20)

    # Models  delta < 5 (fitted ML as in the global model)
    avgmod <- summary(model.avg(get.models(Best20, subset=delta<5)))
    avgmod 

    # Collect and compare models using ANOVA
    basmod <- m1_global
    m1 <-get.models(Best20, subset = 1)[[1]]
    m2 <-get.models(Best20, subset = 2)[[1]]
    m3 <-get.models(Best20, subset = 3)[[1]]
    m4 <-get.models(Best20, subset = 4)[[1]]
    m5 <-get.models(Best20, subset = 5)[[1]]

    # Question:  Describe the variables that were not significant in Model 3. 
                # Do you see any problem with this model?
    summary(m1) 
    summary(m2)
    summary(m3) 
    summary(m4)
    summary(m5) 
    
    # Is the fitted model different from the global model (fitted with all variables)
    
    anova(basmod, m1)	# include p-value here
    anova(basmod, m2)	# include p-value here
    anova(basmod, m3)	# include p-value here 
    anova(basmod, m4)	# include p-value here
    anova(basmod, m5)	# include p-value here

##################### 6. Collect Automatically the Models in a Table #####################################
    
    library(sjlabelled)
    library(sjmisc)
    library(sjstats)
    library(ggeffects)
    library(rlang)
    library(sjPlot)
    

    tab_model(m1, m2, m3, m4, show.se = TRUE, show.aic = TRUE, show.ci = FALSE,
              show.r2 = TRUE, collapse.ci = FALSE, show.stat = TRUE, show.icc = FALSE,
              string.se = "SE", string.p = "P", digits = 3, dv.labels = c("Model 1", "Model 2","Model 3","Model 4"),
              col.order = c("est","se", "ci", "p")) # Ctrl + S to save this table
    
    # Question: What is the model with lowest AIC value?
    
################### 7. Variance inflation factor (VIF) ###################

  # Reference: NANP Nutrition Models Workshop. 2017 ADSA Annual. R.R. White, 2017.  

  # Function to return variance inflation factors
  VIF <- function (fit) {
  v <- vcov(fit)
  nam <- names(fixef(fit))
  ns <- sum(1 * (nam == "Intercept" | nam == "(Intercept)"))
  if (ns > 0) {
    v <- v[-(1:ns), -(1:ns), drop = FALSE]
    nam <- nam[-(1:ns)]
  }
  d <- diag(v)^0.5
  v <- diag(solve(v/(d %o% d)))
  names(v) <- nam
  return(v)
  }

  #Check VIF of the candidate models
  VIF(m1) # fill out he model name here
  VIF(m2)
  VIF(m3)
  VIF(m4)

  # Question: What is the model that we will not evaluate? Why?
  
  
##################### 8. Model Evaluation  #####################################
  
  
  # 1. Run RMSE_CCC Function.R (Open > Ctrl+A > Run)
  # 2. Run Plot Residuals Functions.R (Open > Ctrl+A > Run)
  
  # Read evaluation database
  library(readr)
  de <- read_csv("test database  n= 125.csv")
 
  # Calculate predictions using external (independent) Database
  Obs <- de$An_DMIn

  
  # Fill out model parameters here
  
  PredM1 <- 2.583 + (0.085*de$An_BWm) + (0.007*de$An_LactDays) +
    (-0.198*de$Dt_FA)+(-0.031*de$Dt_Forage) + (0.309*de$Obs_MilkProd)
  
  PredM2 <- 5.226 + (0.007*de$An_LactDays) +
    (-0.199*de$Dt_FA)+(-0.031*de$Dt_Forage) + (0.309*de$Obs_MilkProd) +
    (0.013*de$An_BW)
  
  PredM4 <- 3.117 + (0.091*de$An_BWm) +
    (-0.199*de$Dt_FA)+(-0.030*de$Dt_Forage) + (0.300*de$Obs_MilkProd)

  
  # Call Functions
  library(plyr)
  me1 <- as.data.frame(RMSE(Obs, PredM1))
  me1 <- rename(me1, c("Values" = "Model1"))
  
  me2 <- as.data.frame(RMSE(Obs, PredM2))
  me2 <- rename(me2, c("Values" = "Model2"))
  
  me4 <- as.data.frame(RMSE(Obs, PredM4))
  me4 <- rename(me4, c("Values" = "Model4"))
  
  results <- merge(merge(me1, me2, by=c("Statistic"),all=TRUE), 
              me4, all=TRUE, by=c("Statistic"))
  
  results <- results[c(5,6,9,10,11,4,14,2,3,13,7,8,12,1),] # order lines
  
  
  # Export results for descriptive statistics (it will be saved in the working directory (folder)
  write.csv(results, "Model Evaluation.csv",na="",row.names=FALSE)  

  
  # Question: What is the best Model? Why?
  
  
  # Plots
  library(ggplot2)
  resprplt(de$An_DMIn,? , "Model ?") # fill out the best model here (PredM1, PredM2 or PredM4)
    
  # Do we have slope or mean bias? If yes, how to fix it?
  
