## ====================================================================
## Rumen Model Baldwin, 1995
## From Molly v8 (Windows version) and DigMolly7.csl
## TIME IN DAYS
## POOL SIZES IN KG(Lp,Sp,SOLDM,Ot,Ha,Hb,Pi,As,Mi) OR MOLES
## CONCENTRATIONS IN MOLES/KG or LITER
## MOLECULAR WEIGHTS IN KG/MOLE
## FLUXES IN KG OR MOLES PER DAY
## ====================================================================
#load packages
library(deSolve) 
library(FME) 
library(ggplot2)
library(gridExtra)

# SECTION 1. Initial conditions and parameters
times = seq(0, 30, by = 0.5)

#Parameters, molecular weights kg/mole
parms = c(MWSc=0.171,MWOa=0.134,MWPe=0.187,MWSt=0.162,MWFdLi=0.619,MWFl=0.284,
    MWGy=0.092,MWHc=0.132,MWCe=0.162,MWPs=0.115,MWPi=0.115,MWNn=0.322,MWAc=0.060,
    MWPr=0.074,MWBu=0.088,MWVa=0.102,MWCH=0.121,MWCs=0.180,MWRAa=0.133,MWUr=0.060,
    MWAm=0.017,MWLa=0.090,MWAs=0.084,MWFa=0.256,MWFdFa=0.885,MWCH4=0.016,
    MWMiLi=0.632,MWUrN=0.028,
#MW amino acids in protein (AaMW-18 for Water) kg/mole
    MWAla=0.071,MWAsp=0.115,MWGlu=0.129,MWPro=0.097,MWSer=0.087,MWGly=0.057,MWArg=0.156,MWTyr=0.163,
    MWCys=0.103,MWVal=0.099,MWIle=0.113,MWLeu=0.113,MWThr=0.101,MWHis=0.137,MWLys=0.128,MWTrp=0.186,
    MWPhe=0.147,MWMet=0.131,MWMiAa=0.109,MWPmAa=0.113,MWPbAa=0.110,MWPvAa=0.108,MWLaAa=0.116,
#Conversion of Kg nutrients to molecular weight kg/mole
    OaScSc=0.638,PeScSc=0.914,LiScSc=0.138,LaScSc=0.95,HcCeCe=1.02,
    FdFaSc=0.097,AaFvAc=0.60,AaFvPr=0.60,AaFvBu=0.25,LaAcAc=0.88,LaPrPr=0.12,
#Rate Constants
    KSPP=1.33,KFatHb=0.03,KPiAa=7.0,KFatPi=0.03,KWAP=3.5,KLpSp=4.5,
    KMiHa=1.56,KMiHb=0.41,VMiHa=0.85,VMiHb=0.85,KMiHaF=0.095,KMiHbF=0.041,
    KHaCs=6,VRAaFv=2400,cSAPs=0.002,KRAaFv=0.025,KLaFv=0.5, 
    NnAmAM=3.8,AaFvAm=1.325,KAmabs=12.4,UrAmAM=2.0,VPUNAm=5.67e-2,KPUNAm=0.007,
    KiAm=0.003,RYATP=0.026,KYATAa=0.0001,KFGAm=0.0012,LaFvAT=1.0,KFatFG=0.03,
    AaMiG2=4.97,AmMiG1=7.33,FdLiFl=1.8,FdLiCH=0.133,FdFaFl=3.0,
    FlMiG=0.23,CsFvAT=4.0,AaFvAT=0.97,fSAAs=0.0085,KAsabs=5.8,RLV=100, 
    CsMiG1=6.133,HyMiG1=2.71,CDMiG1=0.518,CsMiG2=2.133,VFaEFF=0.015,MiMaAD=20,
    VCsFv=400,KCsFv=0.0035,AmMiG2=1.12,HyMiG2=-0.42,CDMiG2=-0.05, 
    MiPiPi=0.572,MiNnNN=0.095,MiHaHA=0.212,MiLiLI=0.121,MiLiFa=1.2,MiLiBu=0.5,
    MiLiPr=1.0,MiLiCH=0.8,MiLiGl=0.5,KabsAc=10.5,KabsPr=10.5,KabsBu=10.5,
    KabsLa=0.1,LgDCHa=0.7,DCMiPi=0.75,DCMiLi=0.7,LgDCHb=0.1,LgDCPi=0.4,
    LgDCAs=0.85,LgDCAi=0.10,LgDCFa=0.9,dUREA=0.75,KPUNU=857.1,AmUrUR=0.5,AaFvHy=1.14,
#Aa profiles of proteins
#Fraction (w/w) AA in intact dietary protein. When special proteins, 
#e.g. fishmeal, bloodmeal etc. are added an overlay of this AA composition must be input.
    fPAla=0.057,fPAsp=0.106,fPGlu=0.179,fPPro=0.055,fPSer=0.047,fPGly=0.047,
    fPArg=0.082,fPTyr=0.035,fPCys=0.017,fPVal=0.053,fPIle=0.044,fPLeu=0.077,
    fPThr=0.041,fPHis=0.026,fPLys=0.052,fPTrp=0.014,fPPhe=0.053,fPMet=0.013,
#Composition of microbial(Mi) protein in mole/mole
    fMiAla=0.100,fMiAsp=0.116,fMiGlu=0.117,fMiPro=0.043,fMiSer=0.054,fMiGly=0.089,
    fMiArg=0.036,fMiTyr=0.032,fMiCys=0.011,fMiVal=0.059,fMiIle=0.054,fMiLeu=0.073,
    fMiThr=0.056,fMiHis=0.014,fMiLys=0.072,fMiTrp=0.010,fMiPhe=0.043,fMiMet=0.021,
#Diet kg/kg
#The prefix fD indicates the fraction in feed of each nutrient
#in Kg/Kg dry matter.Codes are soluble carbohydrate(Sc),organic
#acids(Oa),pectin(Pe),lactate(La),lipid(Li),starch
#(St),hemicellulose(Hc),cellulose(Ce),soluble protein(Ps),insoluble
#protein(Pi),non-protein nitrogen(Nn),lignin(Lg),soluble ash(As),
#insoluble ash(Ai),volatile fatty acids as in silage(Ac,Bu),and
#organic matter(OM).Stsol is the fraction of starch which is
#soluble and must be input as it is a physical characteristic of
#feed.Similarly,PSF is the proportion of feed which enters the
#small particle pool directly and is a property of feed.
#50-50 forage concentrate diet
    FdSc=0.06,FdOa=0.05,FdPe=0.06,FdLa=0.0,FdLi=0.04,FdSt=0.23,FdHc=0.09,
    FdCe=0.17,FdPs=0.04,FdPi=0.11,FdNn=0.03,FdLg=0.04,FdAs=0.04,FdAi=0.04,
    FdAc=0.0,FdBu=0.0,FdUr=0.0,StSOL=0.2,PSF=0.4,FdFat=0.0,FpAla=0.057,
    FpAsp=0.106,FpGlu=0.179,FpPro=0.055,FpSer=0.047,FpGly=0.047,FpArg=0.082,
    FpTyr=0.035,FpCys=0.017,FpVal=0.053,FpIle=0.044,FpLeu=0.077,FpThr=0.041,
    FpHis=0.026,FpLys=0.052,FpTrp=0.014,FpPhe=0.053,FpMet=0.013,
#High quality grass hay (#12)
#    FdSc=0.054,FdOa=0.06,FdPe=0.095,FdLa=0.0,FdLi=0.052,FdSt=0.048,FdHc=0.1,
#    FdCe=0.21,FdPs=0.076,FdPi=0.076,FdNn=0.039,FdLg=0.089,FdAs=0.05,FdAi=0.046,
#    FdAc=0.0,FdBu=0.0,FdUr=0.0,StSOL=0.2,PSF=0.4,FdFat=0.0,FpAla=0.057,
#    FpAsp=0.106,FpGlu=0.179,FpPro=0.055,FpSer=0.047,FpGly=0.047,FpArg=0.082,
#    FpTyr=0.035,FpCys=0.017,FpVal=0.053,FpIle=0.044,FpLeu=0.077,FpThr=0.041,
#    FpHis=0.026,FpLys=0.052,FpTrp=0.014,FpPhe=0.053,FpMet=0.013,
#CONSET, > 50% concentrate diet 
#    ScAcAC=0.90,ScPrPR=0.46,ScBuBU=0.32,ScLaLA=0,StAcAC=0.80,StPrPR=0.70,StBuBU=0.25,
#    StLaLA=0,HcPrPR=0.58,HcPrPR=0.40,HcBuBU=0.15,CeAcAC=1.58,CePrPR=0.20,CeBuBU=0.11,
#MIXSET, 50/50 forage concentrate diet
    ScAcAC=1.14,ScPrPR=0.43,ScBuBU=0.21,ScLaLA=0,StAcAC=1.00,StPrPR=0.52,StBuBU=0.24,
    StLaLA=0.0,HcAcAC=1.13,HcPrPR=0.49,HcBuBU=0.19,CeAcAC=1.45,CePrPR=0.20,CeBuBU=0.17,
#FORSET, > 50% forage diet
#    ScAcAC=1.38,ScPrPR=0.40,ScBuBU=0.11,ScLaLA=0,StAcAC=1.20,StPrPR=0.34,StBuBU=0.23,
#    StLaLA=0,HcAcAC=1.14,HcPrPR=0.40,HcBuBU=0.23,CeAcAC=1.32,CePrPR=0.20,CeBuBU=0.24,
#Switches
    DDMIN=22,
    CSG=1,    #If CSG=1 then forage is grass or silage
              #If CSG=0 then forage is a LEGUME (1)
    UT=1.0,   #1 Mcal/mole, 4.184 MJ/mole  
    RUM=0.33, #proportion time spent ruminating 
    MatBW=700,BW=700,BCS=3.2,EBW=620, #BWf
    RpHCON=0  #0 fixed RpH, 1 calculated RpH   
)
#Initial values for state Variables
qinit = c(TDMIN = 0,
         PUN = 20,     #Initial Plasma urea N mole
         RpH = 6.5,    #Initial RpH
         Ha = 1.26,    #Initial starch pool kg
         Lp = 3.9,     #Initial large particle pool kg
         Hc = 0.78,    #Initial hemicellulose kg
         Ce = 1.5,     #Initial cellulose kg
         Hb = 2.4,     #Initial beta hexose kg
         Cs = 0.14,    #Initial soluble carbohydrate kg         
         Mi = 1.78,    #Initial microbes kg
         HaMi = 0.24,  #Initial microbes associated with starch particles in Sp kg
         HbMi = 0.24,  #Initial microbes associated with Hb (beta linked) particles kg 
         Pi = 0.924,   #Initial insoluble protein kg
         Ot = 1.32,    #Initial Other kg
         As = 0.73,    #Initial ash kg 
         Fl = 0.522,   #Initial long chain fatty acids kg
         RAa = 0.073,  #Initial rumen amino acids
         RLa =0.0,     #Initial rumen lactate mole
         Am = 0.11,    #Initial ammonia kg
         RAc = 4.3,    #Initial rumen acetate mole
         RPr = 2.2,    #Initial rumen propionate mole
         RBu = 1.0,    #Initial rumen butyrate mole
         TCH4 = 0,     #Total CH4 production moles
         DCH4 = 0,     #CH4 product formation in rumen mole/d
         AccDEI = 0,   #DEI Mcal/d or MJ/d
         AccMEI = 0,   #MEI Mcal/d or MJ/d
         AccGEI = 0    #GEI Mcal/d or MJ/d
)
# SECTION 2. Derivatives
eqns = function(t, q, parms) {
  with(as.list(c(q, parms)), {
#Rates of Holocellulose (Hc+Ce) change with forage type
    if (CSG <= 0) {KCeCs1 = 9}
    if (CSG <= 0) {KHcCs1 = 9}
    if (CSG > 0) {KCeCs1=6}
    if (CSG > 0) {KHcCs1=6}
#Heats of Combustion HC in Mcal/mole. If UT=4.184, then MJ/mole.
#HCFl is stearate, HCFa = 2.38 is for palmitate
#HCMi and HCLg are in Mcal or MJ/kg
    HMiLi=4.53*UT  #HC microbial lipid
    HCH4=0.211*UT
    HCAc=0.209*UT
    HCPr=0.367*UT
    HCBu=0.524*UT
    HCGl=0.673*UT
    HCGy=0.397*UT
    HCFl=2.657*UT
    HCFa=2.712*UT
    HCCs=0.673*UT
    HCHc=0.562*UT
    HCPs=0.656*UT
    HCNn=1.2*UT
    HCCh=0.4*UT
    HCUr=0.152*UT
    HCOa=0.3365*UT
    HFDLi=5.23*UT
    HCLa=0.336*UT
    HCTg=7.57*UT
    HCLm=1.346*UT
    HCAa=0.627*UT
    HCTp=0.3365*UT
    HCMi=5.35*UT
    HCLg=8.3*UT
    HCAla=0.388*UT
    HCArg=0.80*UT
    HCAsp=0.383*UT
    HCCys=0.532*UT
    HCGlu=0.538*UT
    HCGly=0.233*UT
    HCHis=0.805*UT
    HCIle=0.877*UT
    HCLeu=0.856*UT
    HCLys=0.752*UT
    HCMet=0.752*UT
    HCPro=0.638*UT
    HCPhe=1.140*UT
    HCSer=0.345*UT
    HCThr=0.521*UT
    HCTrp=1.355*UT
    HCTyr=1.060*UT
    HCVal=0.667*UT
#Nutrient Calculations
    NDF = FdHc + FdCe + FdLg
    FdOM=1-FdAi-FdAs
    FdGE=(FdLi*8.45+FdSt*4.154+FdHc*4.26+FdCe*4.154+FdNn*5.7+
            (FdPs+FdPi)*5.7+FdLg*8.3+FdAc*3.48+FdBu*5.95+FdSc*3.94+FdFat*9.63+
            FdOa*2.51+FdPe*3.6+FdLa*3.73)*UT
#Computation of total AA mole/kg protein (TOTAA)and fraction of each AA (mole/mole)
#in protein.Note that the Pi notation which is used to compute AA entry due to diet
#below assumes that the amino acid composition of bypass protein is the same as that 
#of protein in the total diet.
    TotAa = fPAla/MWAla+fPAsp/MWAsp+fPGlu/MWGlu+fPPro/MWPro+fPSer/MWSer+
      fPGly/MWGly+fPArg/MWArg+fPTyr/MWTyr+fPCys/MWCys+fPVal/MWVal+fPIle/MWIle+
      fPLeu/MWLeu+fPThr/MWThr+fPLys/MWLys+fPTrp/MWTrp+fPPhe/MWPhe+FpMet/MWMet
    MWPiAa=1/TotAa
    fPiAla=(fPAla/MWAla)/TotAa
    fPiAsp=(fPAsp/MWAsp)/TotAa
    fPiGlu=(fPGlu/MWGlu)/TotAa
    fPiPro=(fPPro/MWPro)/TotAa
    fPiSer=(fPSer/MWSer)/TotAa
    fPiGly=(fPGly/MWGly)/TotAa
    fPiArg=(fPArg/MWArg)/TotAa
    fPiTyr=(fPTyr/MWTyr)/TotAa
    fPiCys=(fPCys/MWCys)/TotAa
    fPiVal=(fPVal/MWVal)/TotAa
    fPiIle=(fPIle/MWIle)/TotAa
    fPiLeu=(fPLeu/MWLeu)/TotAa
    fPiThr=(fPThr/MWThr)/TotAa
    fPiHis=(fPHis/MWHis)/TotAa
    fPiLys=(fPLys/MWLys)/TotAa
    fPiTrp=(fPTrp/MWTrp)/TotAa
    fPiPhe=(fPPhe/MWPhe)/TotAa
    fPiMet=(fPMet/MWMet)/TotAa
#DMI kg/d
    TDMIN=DDMIN
    FdFvAc=FdAc*DDMIN/MWAc
#Soluble sugar Sc (Oa,Pe,Li,glycerol) kg/kg
    OaSc=FdOa*OaScSc
    PeSc=FdPe*PeScSc
    LiSc=FdLi*LiScSc        
    FatSc=FdFat*FdFaSc       
    FdScT=FdSc+OaSc+PeSc+LiSc+FatSc
    Fdtot=FdSc+FdOa+FdPe+FdLa+FdLi+FdFat+FdSt+FdHc+FdCe+FdLg+FdPs+FdPi+FdNn+FdUr
    +FdAs+FdAi+FdAc+FdBu
#Rumen Large particle Pool kg/kg
    fLp=(FdCe+FdHc+FdLg+FdAi+FdPi)*(1.0-PSF) 
    fLpHc=FdHc*(1.0-PSF)/fLp   
    fLpCe=FdCe*(1.0-PSF)/fLp
    fLpPi=FdPi*(1.0-PSF)/fLp
    fOt=FdLg+FdAi          
    fLpOt=fOt*(1.0-PSF)/fLp  
    fLgOt=FdLg/fOt
    FdLpIN=fLp*DDMIN
    LpSp=KLpSp*Lp*RUM   
    dLp=(FdLpIN-LpSp)/3   
    FdStIn=FdSt*DDMIN     
    FdStCs=FdStIn*StSOL 
    FdStHa=FdStIn-FdStCs  
    ScTCs=FdScT*DDMIN/MWSc
    StCs=FdStCs/MWSt       
#Rumen Small particle pool kg
    Sp=Ha+Hb+Pi+Ot         
    SpPer=Sp/(Sp+Lp)    
#Rumen pH adjustments to fermentation stoichiomentry mole
#Rumen pH >= 6.2
    TVFA=RAc+RPr+RBu     #mole
    cVFA=TVFA/RLV*1000   #mole/L
    ScAc=ScAcAC
    ScPr=ScPrPR
    ScBu=ScBuBU
    ScLa=ScLaLA
    StAc=StAcAC
    StPr=StPrPR
    StBu=StBuBU
    StLa=StLaLA
    cRLa=RLa/RLV
    KHcCs=KHcCs1
    KCeCs=KCeCs1
#RpH adjustment to fiber degradation
    if (RpH < 6.2) {KHcCs=KHcCs1-(KHcCs1*1.875*(6.2-RpH))}
    if (RpH < 6.2) {KCeCs=KCeCs1-(KCeCs1*1.875*(6.2-RpH))}
    if (KHcCs < 0) 0 else KHcCs
    if (KCeCs < 0) 0 else KCeCs
#Rumen acidosis
    if (RpH <= 5.4) {MiMaAD=40}
    if (RpH <= 5.4) {ScAc=0}
    if (RpH <= 5.4) {ScPr=0}
    if (RpH <= 5.4) {ScBu=0}
    if (RpH <= 5.4) {ScLa=2}
    if (RpH <= 5.4) {StAc=0}
    if (RpH <= 5.4) {StPr=0}
    if (RpH <= 5.4) {StBu=0}
    if (RpH <= 5.4) {StLa=2}
#Rumen sub-acidosis
    if (RpH > 5.4 && RpH < 6.2){MiMaAD=MiMaAD+(MiMaAD*((0.8-(RpH-5.4))/0.8))} 
#MiMaAD is the effect of pH on the microbe maintenance requirement mole/kg/d
    if (RpH > 5.4 && RpH < 6.2){ScAc=0.70+((RpH-5.4)/0.8)*(ScAcAC-0.70)} 
    if (RpH > 5.4 && RpH < 6.2){ScPr=0.50+((RpH-5.4)/0.8)*(ScPrPR-0.50)} 
    if (RpH > 5.4 && RpH < 6.2){ScBu=0.40+((RpH-5.4)/0.8)*(ScBuBU-0.40)} 
    if (RpH > 5.4 && RpH < 6.2){StAc=0.66+((RpH-5.4)/0.8)*(StAcAC-0.66)} 
    if (RpH > 5.4 && RpH < 6.2){StPr=0.82+((RpH-5.4)/0.8)*(StPrPR-0.82)} 
    if (RpH > 5.4 && RpH < 6.2){StBu=0.26+((RpH-5.4)/0.8)*(StBuBU-0.26)}
    dRpH=0
#Starch Ha kg/d       
    HaP=Ha*KSPP               
    cMiHa=HaMi/Ha            
    SpHaCs=KHaCs*Ha*cMiHa    
    dHa=(FdStHa-HaP-SpHaCs)/10  
    RDCHa=1.0-HaP/FdStIn        
#Cellulose Ce kg/d
    RCeIN=FdCe*DDMIN
    CeIN=RCeIN*PSF     
    LpCeCe=LpSp*fLpCe 
    cMiHb=HbMi/Hb     
    SpCeCs=KCeCs*(1-(FdFat/FdLi*KFatHb))*Ce*cMiHb 
    CeP=KSPP*Ce      
    RDCCe=1.0-CeP/RCeIN          
    dCe=(CeIN+LpCeCe-SpCeCs-CeP)/10
#Hemicellulose Hc kg/d
    RHcIN=FdHc*DDMIN  
    HcIN=RHcIN*PSF    
    LpHcHc=LpSp*fLpHc                    
    SpHcCs=KHaCs*(1-(FdFat/FdLi*KFatHb))*Hc*cMiHb  
    HcP=KSPP*Hc   
    RDCHc=1.0-HcP/RHcIN              
    dHc=(HcIN+LpHcHc-SpHcCs-HcP)/10    
#Holocellulose Hb (Hc and Ce) kg
    HbIN=CeIN+HcIN       
    LpHbHb=LpCeCe+LpHcHc 
    SpHbCs=SpCeCs+SpHcCs 
    HbP=HcP+CeP          
    RDCHb=1.0-HbP/(DDMIN*(FdHc+FdCe))
    dHb=(HbIN+LpHbHb-SpHbCs-HbP)/10  
#Water dynamics kg, L
    SOLDM=Cs*MWCs+Am*MWAm+RPr*MWPr+RBu*MWBu+RAa*MWRAa+As+Fl*MWFl+RAc*MWAc+RLa*MWLa #1.4
    RDM=Lp+Sp+SOLDM+Mi     
    WaMi=Mi*SOLDM/(RDM-Mi) #microbes in rumen fluid kg
    cMiWa=WaMi/SOLDM       
    RUMVOL=RDM/0.11        
    RUMSA=2.41*(EBW**0.75)*RUM  #rate salivation into rumen L/d, RUM=rumination rate d/d
    EATSA=3.2*DDMIN             #rate salivary secreton during eating L/d      
    SAIN=EATSA+RUMSA            #rate of salivation L/d
#Ha and Hb entry into Cs mole
    HaCs=SpHaCs/MWSt       
    HcCs=SpHcCs/MWHc*0.833 
    CeCs=SpCeCs/MWCe       
    CsP=KWAP*Cs            
    cCs=Cs/RLV            
    CsFv=VCsFv*WaMi/(1.0+KCsFv/cCs) 
    Csin=ScTCs+StCs+HaCs+HcCs+CeCs
    fCsHa=HaCs/Csin                 
    fCsHb=(HcCs+CeCs)/Csin          
#Lignin and Insoluble Ash Ot kg
    LpOt=LpSp*fLpOt                   
    FdOt=DDMIN*fOt*PSF 
    OtP=KSPP*Ot        
    dOt=(LpOt+FdOt-OtP)/3 
#Soluble Ash As kg
    cAs=As/RLV                 
    FdAsAs=FdAs*DDMIN          
    SAAs=fSAAs*SAIN            
    AsP=As*KWAP                
    absRAs=KAsabs*cAs          
    dAs=(FdAsAs+SAAs-AsP-absRAs)/5 
#Microbes Mi kg
    cAm=Am/RLV
    FGAm=1.0/(1.0+KFGAm/cAm)   #factor for implanting effects of low ammonia
#concentrations on microbial growth no units
    FGFa=1+(FdFat/FdLi*KFatFG) #factor for implementing effect of Fa availability
#on microbial growth no units
    RLaFv=KLaFv*cMiWa*RLa
    cRAa=RAa/RLV
    RAaFv=VRAaFv*WaMi/(1.0+KRAaFv/cRAa) 
    FdNnAm=FdNn*DDMIN*NnAmAM/MWNn  #rate of feed entry to Am pool mole/d
    ATPF=CsFv*CsFvAT+RAaFv*AaFvAT+0.76*FdNnAm*AaFvAT+RLaFv*LaFvAT #ATP yield from rumen fermentation mole/d
    ATPM=Mi*MiMaAD  #ATP used by microbes for maintenance mole/d        
    ATPG=ATPF-ATPM  #ATP used by microbes for growth mole/d        
    YATP=0.012+RYATP/(1.0/cRAa)  #microbial growth yield coefficient kg/mole
    MiG=ATPG*YATP*FGAm*FGFa 
    G2=1/(1.0+KYATAa/cRAa)  #proportion of microbial growth dependent on AA and peptides kg/kg
    G1=1.0-G2               #proportion of microbial growth without AA or peptide utilization kg/kg               
    RAaMi=MiG*AaMiG2*G2     
    AmMi=MiG*(AmMiG1*G1+AmMiG2*G2) 
    SpMi=Sp/(RDM-Mi)*Mi     
    cMiSp=SpMi/Sp           
    SpMiP=KSPP*SpMi   
    WaMiP=KWAP*WaMi     
    MiP=SpMiP+WaMiP     
    dMi=(MiG-MiP)/3    
    YATPAP=MiG/ATPF         #apparent microbial growth yield coefficient kg/mole ATP
    CsMi=MiG*(CsMiG1*G1+CsMiG2*G2) 
    dCs=(ScTCs+StCs+HaCs+HcCs+CeCs-CsFv-CsMi-CsP)/400 
    HaMiR=cMiHa*SpHaCs              #microbes released from Sp due to St hydrolysis kg/d
    HbMiR=cMiHb*SpHcCs+cMiHb*SpCeCs 
    HaMiF=VMiHa/(1.0+KMiHaF/Ha/Sp)  #fraction of microbes associated with Ha which
#stay in association with the Sp pool when a kg of Ha is hydrolyzed kg/kg
    HbMiF=VMiHb/(1.0+KMiHbF/Hb/Sp)  
    HaMiP=HaMi*KSPP    #rate of microbe passage in association with St in Sp kg/d
    HbMiP=HbMi*KSPP    
    OtMiP=OtP*cMiSp    
    PiP=KSPP*Pi        
    PiMiP=PiP*cMiSp                  
    SpMiPi=OtMiP+PiMiP+HaMiP+HbMiP  
    CsMiG=MiG*CsFv*CsFvAT/ATPF       
    HaMiG=CsMiG*fCsHa   #rate of microbial growth due to fermentation of hydrolyzed Ha kg/d            
    HbMiG=CsMiG*fCsHb   
    MiHaMi=HaMiF*(HaMiG+HaMiR) #amount of microbes which stay in association with Sp Ha after
#growing there or being released due to hydrolysis kg/d    
    MiHbMi=HbMiF*(HbMiG+HbMiR) 
    SpMiHa=KMiHa*Ha*cMiSp  
    SpMiHb=KMiHb*Hb*cMiSp 
    dHaMi=(SpMiHa+MiHaMi-HaMiP-HaMiR) 
    dHbMi=(SpMiHb+MiHbMi-HbMiP-HbMiR) 
    LpMi=(Lp/(RDM-Mi))*Mi
#Insoluble protein Pi kg 
    FdPiPi=FdPi*DDMIN*PSF        
    LpPiPi=LpSp*fLpPi     
    SpPiAa=KPiAa*(1-(FdFat/FdLi*KFatPi))*Pi*cMiSp 
    TPRTIN=(FdPs+FdPi+FdNn)*DDMIN
    RAaP=KWAP*RAa          
    RDCPRT=(TPRTIN-PiP-(RAaP*MWPiAa))/TPRTIN
    dPi=(FdPiPi+LpPiPi-SpPiAa-PiP)/5   
#Rumen Ammonia Am mole
    FdUrAm=FdUr*DDMIN*UrAmAM/MWUr #rate or feed hydrolysis to yield ammonia kg/mole
    AaAm=RAaFv*AaFvAm  
    PUNVOL= EBW
    dPUN=0  #df equation for plasma urea N in blood mole/d
    cPUN=PUN/PUNVOL         
    SANnAm=cPUN*SAIN*UrAmAM #rate of ammonia formation from salivary urea mole/d
    PUNAm=(VPUNAm*(EBW**0.75)/(1.0+KPUNAm/cPUN+cAm/KiAm))*UrAmAM 
    absRAm=KAmabs*Am 
    dAm=(FdNnAm+AaAm+SANnAm+PUNAm-absRAm-AmMi+FdUrAm)/100 
#Amino acids RAa mole
    FdPsAa=FdPs*DDMIN/MWPiAa 
    PiAa=SpPiAa/MWPiAa                     
    SAPsAa=cSAPs*SAIN   #rate of salivary protein entry to rumen AA pool mole/d
    dRAa=(FdPsAa+PiAa+SAPsAa-RAaFv-RAaMi-RAaP)/250      
#Long chain fatty acids metabolism in gut expressed as palmitate equivalents Fl mole 
    FdFl=FdLi*DDMIN/(MWFdLi)*FdLiFl
    FdFl1=FdFat*DDMIN*FdFaFl/MWFdFa
    FlMi=MiG*FlMiG  
    FaP=KWAP*Fl
    SOLOMP=CsP*MWCs+FaP*MWFl+RAaP*MWRAa    
    dFl=(FdFl+FdFl1-FlMi-FaP)/10    
#Fermentation of Cs in rumen mole
    fStCs=(StCs+HaCs)/Csin
    fScCs=ScTCs/Csin
    fHcCs=HcCs/Csin
    fCeCs=CeCs/Csin                       
    CsFvAc=ScAc*fScCs+StAc*fStCs+HcAcAC*fHcCs+CeAcAC*fCeCs
    CsAc=CsFv*CsFvAc
#RAc mole
    RAaAc=AaFvAc*(RAaFv+(0.76*FdNnAm))
    RLaAc=RLaFv*LaAcAc
    absRAc=KabsAc*RAc
    RAcP=RAc*KWAP
    cRAc=RAc/RLV 
    dRAc=(FdFvAc+CsAc+RAaAc+RLaAc-absRAc-RAcP)/20 
    MPAc=RAc/TVFA
#RPr mole
    absRPr=KabsPr*RPr
    RLaPr=RLaFv*LaPrPr
    RAaPr=AaFvPr*(RAaFv+(0.76*FdNnAm))    
    RPrP=RPr*KWAP
    MPPr=RPr/TVFA
    CsFvPr=ScPrPR*fScCs+StPrPR*fStCs+HcPrPR*fHcCs+CePrPR*fCeCs
    CsPr=CsFv*CsFvPr
    cRPr=RPr/RLV #0.0187
    dRPr=(CsPr+RAaPr+RLaPr-absRPr-RPrP)/20 
#RBu mole   
    CsFvBu=ScBu*fScCs+StBu*fStCs+HcBuBU*fHcCs+CeBuBU*fCeCs
    CsBu=CsFv*CsFvBu
    RAaBu=AaFvBu*(RAaFv+(0.76*FdNnAm))  
    FdFvBu=FdBu*DDMIN/MWBu
    absRBu=KabsBu*RBu
    RBuP=RBu*KWAP
    cRBu=RBu/RLV 
    dRBu=(CsBu+RAaBu+FdFvBu-absRBu-RBuP)/20   
    MPBu=RBu/TVFA
#RLa mole
    CsFvLa=ScLa*fScCs+StLa*fStCs
    CsLa=CsFv*CsFvLa
    FdFvLa=DDMIN*FdLa/MWLa
    RLaP=RLa*KWAP      
    RLaFv=KLaFv*cMiWa*RLa
    absRLa=RLa*KabsLa
    dRLa=CsLa+FdFvLa-RLaP-absRLa-RLaFv
    RpH=(7.20-(VFaEFF*cVFA+0.0015*(cRLa*1000)))*RpHCON
    cVFA=TVFA/RLV*1000
    TVFA=RAc+RPr+RBu
#Lower Gut Functions Lg mole
    LgHaGl=HaP*LgDCHa/MWSt
    MiGl=MiP*MiHaHA*LgDCHa/MWSt
    MiAa=MiP*MiPiPi*DCMiPi/MWMiAa   
    MiLiDg=MiP*MiLiLI*DCMiLi/MWMiLi        
    MiFa=MiLiDg*MiLiFa
    LgFaDg=LgDCFa*FaP
    MiBu=MiLiDg*MiLiBu
    MiPr=MiLiDg*MiLiPr
    MiLgGl=MiLiDg*MiLiGl
    MiCH=(MiP*MiLiLI/MWMiLi)*MiLiCH
    LgHcFv=HcP*LgDCHb/MWHc*0.833 
    LgHcAc=LgHcFv*HcAcAC       
    LgHcPr=LgHcFv*HcPrPR
    LgHcBu=LgHcFv*HcBuBU
    LgCeFv=CeP*LgDCHb/MWCe
    LgCeAc=LgCeFv*CeAcAC
    LgCePr=LgCeFv*CePrPR
    LgCeBu=LgCeFv*CeBuBU
    LgPiAa=PiP*LgDCPi/MWPiAa      
    LgAs=LgDCAs*AsP                                     
    LgAi=LgDCAi*OtP*FdAi/fOt
#Feces Fe kg
    FeHa=HaP*(1.0-LgDCHa)         
    FeHb=HbP*(1.0-LgDCHb)         
    FeHc=HcP*(1.0-LgDCHb)         
    FeCe=CeP*(1.0-LgDCHb)         
    fMiPi=MiP*MiPiPi*(1.0-DCMiPi) 
    fMiNn=MiP*MiNnNN                      
    fMiLi=MiP*MiLiLI*(1.0-DCMiLi)
    FeFa=FaP*(1-LgDCFa)*MWFl
    FePi=PiP*(1.0-LgDCPi)
    FeASH=AsP*(1.0-LgDCAs)+OtP*FdAi/fOt*(1.0-LgDCAi)
    FeMiHa=MiP*MiHaHA*(1.0-LgDCHa)
    FeLg=OtP*FdLg/fOt
    FePiT=fMiPi+fMiNn+FePi
    FeCH=((FdLi*DDMIN/MWFdLi)*FdLiCH)*MWCH+MiCH*DCMiLi*MWCH #CH=choline
    FeOM=FeHa+FeHb+FePiT+fMiLi+FeLg+FeCH+FeMiHa+FeFa
    FeENG=(FeHa*4.154+FeHb*4.154+(fMiPi+FePi)*5.7+fMiLi*7.2+
           FeLg*8.3+FeCH*3.31+fMiNn*5.7+FeMiHa*4.154+FeFa*9.53)*UT #fecal energy Mcal or MJ
    FeDM=FeOM+FeASH
    FdOMIN=DDMIN*FdOM
    SOLDMP=SOLOMP+AsP
    TOMP=SOLOMP+HaP+HbP+PiP+OtP*fLgOt
    TTOMP=TOMP+MiP            
    RDCOM=1.0-TOMP/FdOMIN   
    RDCOMa=1.0-TTOMP/FdOMIN   
    RDPRT=(TPRTIN-PiP-(RAaP*MWPiAa)-(MiP*MiPiPi))/TPRTIN 
#Digestion coefficients kg/kg 
    TStIN=DDMIN*FdSt
    DCHa=(TStIN-FeHa)/TStIN
    DCHc=(RHcIN-FeHc)/RHcIN     
    DCCe=(RCeIN-FeCe)/RCeIN     
    DCHb=(HbIN-FeHb)/HbIN
    DCLg=((FdLg*DDMIN)-FeLg)/(FdLg*DDMIN)
    DCPRT=(TPRTIN-FePiT)/TPRTIN     
    DCOM=(FdOMIN-FeOM)/FdOMIN     
    DCDM=1.0-FeDM/DDMIN           
#Energy Mcal/d or MJ/d
    FdGEIN=FdGE*DDMIN  
    appDE=(FdGEIN-FeENG)/FdGEIN
    DEI=FdGEIN-FeENG  
    AccDEI=DEI
    DE=DEI/DDMIN
    EUr=dUREA*HCUr
    GE=FdGEIN/DDMIN
#Interface with Mollyr - Nutrient abs
    absGl=LgHaGl+CsP+MiGl+MiLgGl
    TabsAa=MiAa+LgPiAa+RAaP
    absAc=absRAc+LgHcAc+LgCeAc+RAcP 
    absPr=absRPr+MiPr+LgHcPr+LgCePr+RPrP
    absBu=absRBu+LgHcBu+LgCeBu+MiBu+RBuP
    absAm=absRAm  
    absFa=(MiFa+LgFaDg)*MWFl/MWFa
    absAs=absRAs+LgAs+LgAi
    absLa=absRLa+RLaP
    absAcE=absAc*HCAc
    absPrE=absPr*HCPr
    absBuE=absBu*HCBu
    absFaE=absFa*HCFa
    absAaE=TabsAa*HCAa
    absGlE=absGl*HCGl
    absLaE=absLa*HCLa
    absE=absAcE+absPrE+absBuE+absFaE+absAaE+absGlE+absLaE
    TDE=absE/FdGEIN
#Methane CH4 mole/d or Mcal/d, MJ/d
    dCsFvH=CsFvAc*2.0-CsFvPr*1.0+CsFvBu*2.0
    dCsHy=CsFv*dCsFvH
    dRAaHy=RAaFv*AaFvHy
    dHyMi=HyMiG1*G1+HyMiG2*G2
    dHyFlF=FdFl*2.0
    dTHy=dCsHy+dRAaHy-dHyMi-dHyFlF+2.0*RLaAc-RLaPr #daily net balance of H formation
#from fermentation, hydrogen use in microbial growth, etc. mole/d
    dDCH4=dTHy/4.0   #df equation for CH4 product formation in rumen mole/d
    dTCH4=dDCH4      #df equation for summing CH4 production over time mole/d
    CH4E=dTCH4*HCH4  #energy loss in CH4 
    CH4EFd=CH4E/DDMIN  #CH4 energy loss per unit feed intake Mcal/kg 
    fCH4E=CH4E/FdGEIN  #CH4 energy as a fraction of GE intake Mcal/Mcal
    fCH4DE=CH4E/DEI    #CH4 energy as a fraction of DEI Mcal/Mcal    
    MEI=FdGEIN-CH4E-EUr-FeENG 
    ME=MEI/DDMIN
    AccMEI=MEI
    estNEl=ME*0.64
    HFERM=FdGEIN-absE-FeENG-CH4E-EUr
    corMEI=MEI-HFERM   #corrected ME intake Mcal/d
    corME=corMEI/DDMIN #corrected ME Mcal/kg
    CH4KgY=TCH4*0.016  #CH4 production in kg/yr
    TCH4E=TCH4*HCH4    #total CH4 energy production mole/d 
    netME=AccMEI/TDMIN 
    AccGEI=FdGEIN
    
    CH4GEI=TCH4E/AccGEI #CH4 energy lost per unit gross energy intake Mcal/Mcal
    CH4DEI=TCH4E/AccDEI #CH4 production per unit DEI Mcal/Mcal
    CH4MEI=TCH4E/AccMEI #CH4 production per unit MEI Mcal/Mcal

 res = c(TDMIN,dPUN,dRpH,dHa,dLp,dHc,dCe,dHb,dCs,dMi,dHaMi,dHbMi,dPi,dOt,dAs,dFl,dRAa,dRLa,dAm,dRAc,dRPr,
         dRBu,dTCH4,dDCH4,AccDEI,AccMEI,AccGEI)
 list(res) 
})
}

# SECTION 3. Results integrate numerically
out1 = rk(qinit, times, eqns, parms, method = "rk4")
df = as.data.frame(out1)

#Add non-integrated variables to dataframe
#Mole % VFA
df$TVFA=df$RAc+df$RPr+df$RBu    
df$MPAc=df$RAc/df$TVFA
df$MPPr=df$RPr/df$TVFA
df$MPBu=df$RBu/df$TVFA
df$cVFA=df$TVFA/parms["RLV"]*1000
df$cRLa=df$RLa/parms["RLV"]
df$RpH1=(7.20-(0.015*df$cVFA+0.0015*(df$cRLa*1000)))

#NRG
df$FdGE=(parms["FdLi"]*8.45+parms["FdSt"]*4.154+parms["FdHc"]*4.26
         +parms["FdCe"]*4.154+parms["FdNn"]*5.7+(parms["FdPs"]+parms["FdPi"])*5.7
         +parms["FdLg"]*8.3+parms["FdAc"]*3.48+parms["FdBu"]*5.95+parms["FdSc"]*3.94
         +parms["FdFat"]*9.63+parms["FdOa"]*2.51+parms["FdPe"]*3.6+parms["FdLa"]*3.73)*parms["UT"]
df$HaP=df$Ha*parms["KSPP"]  
df$FeHa=df$HaP*(1.0-parms["LgDCHa"])
df$HcP=parms["KSPP"]*df$Hc
df$FeHc=df$HcP*(1.0-parms["LgDCHb"])    
df$CeP=parms["KSPP"]*df$Ce
df$FeCe=df$CeP*(1.0-parms["LgDCHb"])  
df$HbP=df$HcP+df$CeP
df$FeHb=df$HbP*(1.0-parms["LgDCHb"])         
df$Sp=df$Ha+df$Hb+df$Pi+df$Ot #6
df$SOLDM=df$Cs*parms["MWCs"]+df$Am*parms["MWAm"]+df$RPr*parms["MWPr"]+df$RBu*parms["MWBu"]+df$RAa/10*parms["MWRAa"]+df$As+df$Fl*parms["MWFl"]+df$RAc*parms["MWAc"]+df$RLa*parms["MWLa"]
df$RDM=df$Lp+df$Sp+df$SOLDM+df$Mi  
df$SpMi=df$Sp/(df$RDM-df$Mi)*df$Mi   
df$SpMiP=parms["KSPP"]*df$SpMi 
df$WaMi=df$Mi*df$SOLDM/(df$RDM-df$Mi) 
df$WaMiP=parms["KWAP"]*df$WaMi       
df$MiP=df$SpMiP+df$WaMiP  
df$fMiPi=df$MiP*parms["MiPiPi"]*(1.0-parms["DCMiPi"]) 
df$fMiNn=df$MiP*parms["MiNnNN"]                      
df$fMiLi=df$MiP*parms["MiLiLI"]*(1.0-parms["DCMiLi"])
df$FaP=parms["KWAP"]*df$Fl
df$FeFa=df$FaP*(1-parms["LgDCFa"])*parms["MWFl"]
df$PiP=parms["KSPP"]*df$Pi 
df$FePi=df$PiP*(1.0-parms["LgDCPi"])
df$AsP=df$As*parms["KWAP"]
df$OtP=parms["KSPP"]*df$Ot
df$fOt=parms["FdLg"]+parms["FdAi"]       
df$FeASH=df$AsP*(1.0-parms["LgDCAs"])+df$OtP*parms["FdAi"]/df$fOt*(1.0-parms["LgDCAi"])
df$FeMiHa=df$MiP*parms["MiHaHA"]*(1.0-parms["LgDCHa"])
df$FeLg=df$OtP*parms["FdLg"]/(parms["FdLg"]+parms["FdAi"])
df$FePiT=df$fMiPi+df$fMiNn+df$FePi
df$FeCH=((parms["FdLi"]*parms["DDMIN"]/parms["MWFdLi"])*parms["FdLiCH"])*parms["MWCH"]
         +parms["MiCH"]*parms["DCMiLi"]*parms["MWCH"]
df$FeOM=df$FeHa+df$FeHb+df$FePiT+df$fMiLi+df$FeLg+df$FeCH+df$FeMiHa+df$FeFa
df$FeENG=(df$FeHa*4.154+df$FeHb*4.154+(df$fMiPi+df$FePi)*5.7+df$fMiLi*7.2+
         df$FeLg*8.3+df$FeCH*3.31+df$fMiNn*5.7+df$FeMiHa*4.154+df$FeFa*9.53)*parms["UT"]
df$EUr=parms["dUREA"]*0.152*parms["UT"]
df$HCH4=0.211*parms["UT"]
df$TCH4E=df$TCH4*df$HCH4
df$OaSc=parms["FdOa"]*parms["OaScSc"]
df$PeSc=parms["FdPe"]*parms["PeScSc"]
df$LiSc=parms["FdLi"]*parms["LiScSc"]        
df$FatSc=parms["FdFat"]*parms["FdFaSc"]       
df$FdScT=parms["FdSc"]+df$OaSc+df$PeSc+df$LiSc+df$FatSc
df$ScTCs=df$FdScT*parms["DDMIN"]/parms["MWSc"]
df$FdStIn=parms["FdSt"]*parms["DDMIN"]    
df$FdStCs=df$FdStIn*parms["StSOL"]
df$StCs=df$FdStCs/parms["MWSt"]
df$cMiHa=df$HaMi/df$Ha   
df$SpHaCs=parms["KHaCs"]*df$Ha*df$cMiHa
df$HaCs=df$SpHaCs/parms["MWSt"]
df$SpHcCs=parms["KHaCs"]*(1-(parms["FdFat"])/(parms["FdLi"])*(parms["KFatHb"]))*df$Hc
df$HcCs=df$SpHcCs/parms["MWHc"]*0.833 
df$SpCeCs=6*(1-(parms["FdFat"]/parms["FdLi"]*parms["KFatHb"]))*df$Ce
df$CeCs=df$SpCeCs/parms["MWCe"]
df$Csin=df$ScTCs+df$StCs+df$HaCs+df$HcCs+df$CeCs 
df$fScCs=df$ScTCs/df$Csin
df$fStCs=(df$StCs+df$HaCs)/df$Csin
df$fHcCs=df$HcCs/df$Csin
df$fCeCs=df$CeCs/df$Csin 
df$CsFvAc=parms["ScAcAC"]*df$fScCs+parms["StAcAC"]*df$fStCs+parms["HcAcAC"]*df$fHcCs+parms["CeAcAC"]*df$fCeCs
df$CsFvPr=parms["ScPrPR"]*df$fScCs+parms["StPrPR"]*df$fStCs+parms["HcPrPR"]*df$fHcCs+parms["CePrPR"]*df$fCeCs
df$CsFvBu=parms["ScBuBU"]*df$fScCs+parms["StBuBU"]*df$fStCs+parms["HcBuBU"]*df$fHcCs+parms["CeBuBU"]*df$fCeCs
df$dCsFvH=df$CsFvAc*2.0-df$CsFvPr*1.0+df$CsFvBu*2.0
df$cCs=df$Cs/parms["RLV"]
df$CsFv=parms["VCsFv"]*df$WaMi/(1.0+parms["KCsFv"]/df$cCs) 
df$dCsHy=df$CsFv*df$dCsFvH
df$cRAa=df$RAa/parms["RLV"]
df$RAaFv=parms["VRAaFv"]*df$WaMi/(1.0+parms["KRAaFv"]/df$cRAa)
df$dRAaHy=df$RAaFv*parms["AaFvHy"]
df$G2=1/(1.0+parms["KYATAa"]/df$cRAa)
df$G1=1.0-df$G2                      
df$dHyMi=parms["HyMiG1"]*df$G1+parms["HyMiG2"]*df$G2
df$FdFl=parms["FdLi"]*parms["DDMIN"]/(parms["MWFdLi"])*parms["FdLiFl"]
df$dHyFlF=df$FdFl*2.0
df$cMiWa=df$WaMi/df$SOLDM
df$RLaFv=parms["KLaFv"]*df$cMiWa*df$RLa
df$RLaAc=df$RLaFv*parms["LaAcAc"]
df$RLaPr=df$RLaFv*parms["LaPrPr"]
df$dTHy=df$dCsHy+df$dRAaHy-df$dHyMi-df$dHyFlF+2.0*df$RLaAc-df$RLaPr
df$dTCH4=df$dTHy/4.0
df$CH4E=df$dTCH4*df$HCH4
df$FdGEIN=df$FdGE*parms["DDMIN"]
df$DEI=df$FdGEIN-df$FeENG
df$MEI=df$DEI-df$CH4E-df$EUr

#Digestion Coefficients
df$TStIN=parms["DDMIN"]*parms["FdSt"]
df$RHcIN=(parms["FdHc"])*(parms["DDMIN"])
df$HcIN=df$RHcIN*(parms["PSF"])
df$RCeIN=parms["FdCe"]*parms["DDMIN"]
df$CeIN=df$RCeIN*parms["PSF"]
df$HbIN=df$CeIN+df$HcIN
df$TPRTIN=(parms["FdPs"]+parms["FdPi"]+parms["FdNn"])*parms["DDMIN"]
df$FdOM=1-parms["FdAi"]-parms["FdAs"]
df$FdOMIN=parms["DDMIN"]*df$FdOM
df$FeDM=df$FeOM+df$FeASH
df$DCHa=(df$TStIN-df$FeHa)/df$TStIN
df$DCHc=(df$RHcIN-df$FeHc)/df$RHcIN
df$DCCe=(df$RCeIN-df$FeCe)/df$RCeIN     
df$DCHb=(df$HbIN-df$FeHb)/df$HbIN
df$DCLg=((parms["FdLg"]*parms["DDMIN"])-df$FeLg)/(parms["FdLg"]*parms["DDMIN"])
df$DCPRT=(df$TPRTIN-df$FePiT)/df$TPRTIN
df$DCOM=(df$FdOMIN-df$FeOM)/df$FdOMIN
df$DCDM=1.0-df$FeDM/parms["DDMIN"]           

#Molly Interface
#df$LgHaGl=df$HaP*parms["LgDCHa"]/parms["MWSt"]
#df$CsP=parms["KWAP"]*df$Cs
#df$MiGl=df$MiP*parms["MiHaHA"]*parms["LgDCHa"]/parms["MWSt"]
#df$MiLiDg=df$MiP*parms["MiLiLI"]*parms["DCMiLi"]/parms["MWMiLi"]       
#df$MiLgGl=df$MiLiDg*parms["MiLiGl"]
#df$absGl=df$LgHaGl+df$CsP+df$MiGl+df$MiLgGl
#df$MiAa=df$MiP*parms["MiPiPi"]*parms["DCMiPi"]/parms["MWMiAa"]  
#df$RAaP=parms["KWAP"]*df$RAa
#df$TotAa = parms["fPAla"]/parms["MWAla"]+parms["fPAsp"]/parms["MWAsp"]
#        +parms["fPGlu"]/parms["MWGlu"]+parms["fPPro"]/parms["MWPro"]
#        +parms["fPSer"]/parms["MWSer"]+parms["fPGly"]/parms["MWGly"]
#        +parms["fPArg"]/parms["MWArg"]+parms["fPTyr"]/parms["MWTyr"]
#        +parms["fPCys"]/parms["MWCys"]+parms["fPVal"]/parms["MWVal"]
#        +parms["fPIle"]/parms["MWIle"]+parms["fPLeu"]/parms["MWLeu"]
#        +parms["fPThr"]/parms["MWThr"]+parms["fPLys"]/parms["MWLys"]
#        +parms["fPTrp"]/parms["MWTrp"]+parms["fPPhe"]/parms["MWPhe"]
#        +parms["FpMet"]/parms["MWMet"]
#df$MWPiAa=1/df$TotAa
#df$LgPiAa=df$PiP*parms["LgDCPi"]/df$MWPiAa   
#df$TabsAa=df$MiAa+df$LgPiAa+df$RAaP
#df$absRAc=parms["KabsAc"]*df$RAc
#df$RAcP=df$RAc*parms["KWAP"]
#df$LgHcFv=df$HcP*parms["LgDCHb"]/parms["MWHc"]*0.833 
#df$LgHcAc=df$LgHcFv*parms["HcAcAC"]
#df$LgCeFv=df$CeP*parms["LgDCHb"]/parms["MWCe"]
#df$LgCeAc=df$LgCeFv*parms["CeAcAC"]
#df$absAc=df$absRAc+df$LgHcAc+df$LgCeAc+df$RAcP 
#df$absRPr=parms["KabsPr"]*df$RPr
#df$MiPr=df$MiLiDg*parms["MiLiPr"]
#df$LgHcPr=df$LgHcFv*parms["HcPrPR"]
#df$LgCeFv=df$CeP*parms["LgDCHb"]/parms["MWCe"]
#df$LgCePr=df$LgCeFv*parms["CePrPR"]
#df$RPrP=df$RPr*parms["KWAP"]
#df$absPr=df$absRPr+df$MiPr+df$LgHcPr+df$LgCePr+df$RPrP
#df$LgHcBu=df$LgHcFv*parms["HcBuBU"]
#df$absRBu=parms["KabsBu"]*df$RBu
#df$RBuP=df$RBu*parms["KWAP"]
#df$LgCeBu=df$LgCeFv*parms["CeBuBU"]
#df$MiBu=df$MiLiDg*parms["MiLiBu"]
#df$absBu=df$absRBu+df$LgHcBu+df$LgCeBu+df$MiBu+df$RBuP
#df$absAm=parms["KAmabs"]*df$Am
#df$MiFa=df$MiLiDg*parms["MiLiFa"]
#df$LgFaDg=parms["LgDCFa"]*df$FaP
#df$absFa=(df$MiFa+df$LgFaDg)*parms["MWFl"]/parms["MWFa"]
#df$cAs=df$As/parms["RLV"]
#df$absRAs=parms["KAsabs"]*df$cAs
#df$LgAs=parms["LgDCAs"]*df$AsP                                     
#df$LgAi=parms["LgDCAi"]*df$OtP*parms["FdAi"]/df$fOt
#df$absAs=df$absRAs+df$LgAs+df$LgAi
#df$RLaP=df$RLa*parms["KWAP"]      
#df$absRLa=df$RLa*parms["KabsLa"]
#df$absLa=df$absRLa+df$RLaP
#df$absAcE=df$absAc*0.209*parms["UT"]
#df$absPrE=df$absPr*0.367*parms["UT"]
#df$absBuE=df$absBu*0.524*parms["UT"]
#df$absFaE=df$absFa*2.712*parms["UT"]
#df$absAaE=df$TabsAa*0.627*parms["UT"]
#df$absGlE=df$absGl*0.673*parms["UT"]
#df$absLaE=df$absLa*0.336*parms["UT"]
#df$absE=df$absAcE+df$absPrE+df$absBuE+df$absFaE+df$absAaE+df$absGlE+df$absLaE
#df$TDE=df$absE/df$FdGEIN

#see what you have in the first rows
#head(df, 70)
#see what you have in the last rows
#tail(df, 70)

# plots 
colnames(df)
matplot(x=df$time, y=df[,30:32], type = "l", lwd = 3, col = c(31,32,33), ylim=c(0,1),
        xlab="Time,d", ylab="mole %")
legend("topright", c("MPAc", "MPPr", "MPBu"), fill=c(31,32,33), cex = .8)

matplot(x=df$time, y=df[,115:118], type = "l", lwd = 3, col = c(116,117,118,119), ylim=c(0,100),
        xlab="Time,d", ylab="Mcal/d")
legend("topright", c("CH4E","FdGEIN","DEI","MEI"), fill=c(116,117,118,119), cex = .8)

matplot(x=df$time, y=df[,129:132], type = "l", lwd = 3, col = c(130,131,132,133), ylim=c(-1,1),
        xlab="Time,d", ylab="kg/kg")
legend("bottomright", c("DCHa", "DCHc", "DCCe", "DCHb"), fill=c(130,131,132,133), cex = .8)

matplot(x=df$time, y=df[,133:136], type = "l", lwd = 3, col = c(134,135,136,137), ylim=c(0,1),
        xlab="Time,d", ylab="kg/kg")
legend("topright", c("DCLg", "DCPRT", "DCOM", "DCDM"), fill=c(134,135,136,137), cex = .8)

matplot(x=df$time, y=df[,34], type = "l", lwd = 3, col = c(35), ylim=c(0,5),
        xlab="Time,d", ylab="mole/L")
legend("topright", c("cRLa"), fill=c(35), cex = .8)

matplot(x=df$time, y=df[,35], type = "l", lwd = 3, col = c(36), ylim=c(0,10),
        xlab="Time,d", ylab="pH")
legend("topright", c("RpH"), fill=c(36), cex = .8)




